function model = milesparafac(x,weights,Fac,beta,oldloads);

%MILES-PARAFAC for fitting maximum likelihood PARAFAC
%
% INPUT:
% x         Data matrix of size I x J x K
% 
% weights   matrix of size IJK x IJK holding the weights. For computational
%           reasons, it is advisable to provide the weights in a sparse 
%           matrix if possible.
%           
%           If the weights provided are only of size JKxJK, they are assumed to be
%           independent and identical across rows.
%
% Fac       Number of factors to use
% 
% optional input
% beta      Largest eigenvalue of the matrix weights'*weights. If supplied, the
%           algorithm will skip the (sometimes time-consuming) step of finding
%           this
% 
% oldloads  optional set of parafac loadings used for initializing PARAFAC
% 
% OUTPUT:
% model     struct array holding the outputs of the least squares algorithm.
% 
% I/O: model = miles(x,weights,Fac);
%
%
% Copyright Rasmus Bro, Nikos Sidiropoulos, Age Smilde, 2001
%           rb@kvl.dk
%
% If you use this m-file please supply reference to 
%
% Rasmus Bro, Nicholas D. Sidiropoulos, Age K. Smilde
% Maximum likelihood fitting using ordinary least squares algorithms
% Journal of Chemometrics, 2001, Submitted


% 18-9 Missing auxiliary functions have been added


% Initialization
ConvCrit = 1e-8;
ShowFit  = 100; 
MaxIt    = 5000;
if nargin<4,Cent = 0;end
if nargin<5,,FixOffset=0;FixP=0;end
DimX = size(x);
wtw = weights'*weights;
if size(wtw,1) == prod(DimX(2:3))
   SampleInd = 1;
else
   SampleInd = 0;
end
model    = x;
residual = zeros(size(x));

% Loss
Loss0 = mlloss(x,wtw,DimX,SampleInd);
Loss     = Loss0;
OldLoss  = Loss*2;

Count    = 0;
LineSearchDelta = 20;

clear weights
if nargin<4|beta==0
   OPTIONS.disp = 0;
   beta = eigs(wtw,1,'LM',OPTIONS);
end


if nargin<5
   modells = parafac(x,Fac,[],[],[],[1 0],0);
   MODEL.loads = modells.loads;
else
   MODEL.loads = oldloads;
end


lort=[];

while abs(Loss-OldLoss)/OldLoss > ConvCrit & Count < MaxIt
   
   Count = Count + 1;
   OldLoss = Loss;
   
   if Count>1
      OldQ = Q;
   end
   
   
   if SampleInd
      q = model;
      for i = 1:DimX(1)
         q(i,:) = q(i,:) + ((1/beta)*wtw*residual(i,:)')';
      end
   else
      q = model(:) + (1/beta)*wtw*residual(:);
   end
   Q = reshape(q,DimX);
   
   % Quadratic line search
   if ~rem(Count,4)&Count>20
      Del1 = LineSearchDelta;Del2 = LineSearchDelta*2;  % Base the linesearch on old value as well as old optimal delta (+ 2*delta)
      DeltaQ = Q-OldQ;
      loss1 = Loss;
      modells = parafac(Q+Del1*DeltaQ,Fac,[],[],MODEL.loads,[1 0],0);
      modells = outerm(modells.loads);
      residualls = x - modells;
      loss2 = mlloss(residualls,wtw,DimX,SampleInd);
      %loss2 = residualls(:)'*wtw*residualls(:);
      modells = parafac(Q+Del2*DeltaQ,Fac,[],[],MODEL.loads,[1 0],0);
      modells = outerm(modells.loads);
      residualls = x - modells;
      loss3 = mlloss(residualls,wtw,DimX,SampleInd);
      %loss3 = residualls(:)'*wtw*residualls(:);
      beta23 = Del1^2-Del2^2;beta31=Del2^2;beta12=-Del1^2;gamma23=Del1-Del2;gamma31=Del2;gamma12=-Del1;
      
      % 'Optimal' delta
      LineSearchDelta = .5 *(beta23*loss1+beta31*loss2+beta12*loss3) / (gamma23*loss1+gamma31*loss2+gamma12*loss3);
      if isnan(LineSearchDelta)|isinf(LineSearchDelta)
         LineSearchDelta = 10;
      end
      
      modells = parafac(Q+LineSearchDelta*DeltaQ,Fac,[],[],MODEL.loads,[1 0],0);
      modells = outerm(modells.loads);
      residualls = x - modells;
      Lossls = mlloss(residualls,wtw,DimX,SampleInd);
      %Lossls = residualls(:)'*wtw*residualls(:);
      
      if Lossls<Loss
         Loss = Lossls;
         model = modells;
         if SampleInd
            model = modells;
            q = modells;
            for i = 1:DimX(1)
               if any(isnan(residual(i,:)))
                  id = find(~isnan(residual(i,:)));
                  bwr = ((1/beta)*wtw(:,id)*residual(i,id)')';
               else
                  bwr = ((1/beta)*wtw*residual(i,:)')';
               end
               q(i,:) = q(i,:) + bwr;
            end
         else
            if any(isnan(residual(:)))
               id = find(~isnan(residual(:)));
               q(id) = modells(id) + (1/beta)*wtw(:,id)*residual(id);
            else
               q = modells(:) + (1/beta)*wtw*residual(:);
            end
         end
         Q = reshape(q,DimX);
      end
   end
   model = parafac(Q,Fac,[],[],MODEL.loads,[1 0],0);
   MODEL = model;
   model = outerm(model.loads);
   residual = x - model;
   
   Loss = mlloss(residual,wtw,DimX,SampleInd);
   %Loss = residual(:)'*wtw*residual(:);
   lort=[lort Loss]; 
   if rem(Count,ShowFit)==0
      fprintf(' %8.0f       %12.10f        %12.10f       \n',Count,Loss,100*(1-Loss/Loss0));
      if Count>1,   plot(log(lort(2:end))),drawnow,end
   end
   
end
fprintf(' %8.0f       %12.10f        %12.10f       \n',Count,Loss,100*(1-Loss/Loss0));

model = MODEL;
model.loss = Loss;



function loss = mlloss(x,wtw,DimX,SampleInd)

% Loss
if SampleInd 
   loss = 0;
   for i = 1:DimX(1)
      if any(isnan(x(i,:))) % Missing
         id = find(~isnan(x(i,:)));
         l = x(i,id)*wtw(id,id)*x(i,id)';
      else
         l = x(i,:)*wtw*x(i,:)';
      end
      loss = loss + l;
   end
else
   loss = x(:)'*wtw*x(:);
end














function model = parafac(x,nocomp,scl,tol,x0,options,plots,constraints,weights);

% INITIALIZATION OF ALGORITHM
Show = 100; % How often are fit values shown
xsize = size(x);
order = ndims(x);
if (nargin < 3 | ~strcmp(class(scl),'cell'))
	scl = cell(1,order);
end
for ii=1:order   %added 3/1/00 nbg
	if isempty(scl{ii})|(length(scl{ii})~=xsize(ii))
		scl{ii} = 1:xsize(ii);
	end
end
standardtol = [1e-6 1e-6 10000];
if (nargin < 4 | length(tol)==0)
	tol = standardtol;
else % Changed error rb 11/03/00
	tol(find(tol==0)) = standardtol(find(tol==0));
end
if nargin < 6
	options = [1 1];
end
if length(options)<2
	options = [options ones(1,2-length(options))];
end
ls = options(1); %For line-searh
DumpToScreen = options(2);
if nargin < 7|length(plots)==0
	plots = 1;
end
if nargin < 8
	constraints = zeros(1,order);
end
if length(constraints)~=order
	constraints = zeros(1,order);
end

if nargin < 9 
	DoWeight = 0;
else
	if length(size(weights))~=order 
		DoWeight = 0;
	elseif all(size(weights)==xsize)
		DoWeight = 1;
		WMax     = max(abs(weights(:)));
		W2       = weights.*weights;
	else
		DoWeight = 0;
	end
end

%Iterative preproc
iteroffsets = 0;
if exist('iteroff')==1
	if iscell(iteroff)
		iteroffsets = 1;
	elseif isnumeric(iteroff) % Then convert to cell
		hh = iteroff;
		clear iteroff
		iteroff{1} = hh;
		iteroffsets = 1;
	elseif ischar(iteroff)
		if strcmp(lower(iteroff(1:2)),'he')
			clear iteroff
			button = questdlg('Do you want help to incorporate offsets?','Offsets','Yes','No','Cancel','Yes');
			if strcmp(button,'Cancel')
				break
			elseif strcmp(button,'No')
				iteroffsets = 0;
			else
				Continue = 1;
				while Continue
					if Continue==1
						questdlg('For each type of offset, you input the number of the modes across which the offsets are constant. E.g. for a three-way array where you want ordinary offsets that are constant across the first mode (~average of each column); then you input the offset as [1], meaning the the offset is constant across the first mode. In this case the offsets will therefore be a matrix of dimension J x K (J second mode dimension, K third mode dimension). If you want an offset that is constant across all three modes, you input [1 2 3], implying that one offset is estimated','Offsets - how to do it','OK','OK');
					end
					prompt={['Enter the definition of offset # ',num2str(Continue)]};
					def={'[1]'};
					dlgTitle='Offset definition';
					lineNo=1;
					% Ask for offset
					Def=inputdlg(prompt,dlgTitle,lineNo,def);
					% If cancel is hit instead of OK
					if isempty(Def)
						hh=questdlg('Offsets across the first mode will be assumed','Drop offsets?','OK','Stop PARAFAC','OK');
						if strcmp(hh,'Stop PARAFAC')
							error(' PARAFAC stopped')
						else
							Def{1} = '[1]';
						end
					end
					% Verify the offset and show how it's implemented
					hh = eval(Def{1});
					txt = ['[',num2str(hh(1))];
					for i = 2:length(hh);
						txt = [txt,' ',num2str(hh(i))];
					end
					txt = [txt,']'];
					prompt={['Offset # ',num2str(Continue),' is defined as: iteroff{',num2str(Continue),'} = ',txt,';']};
					dlgTitle='Learn how to use the offset input <iteroff>';
					% Ask for offset
					questdlg(prompt,dlgTitle,'OK','Cancel','OK');
					iteroff{Continue} = eval(Def{1});
					Continue = Continue + 1;
					answer=questdlg('Do you want to incorporate other offsets?','More?','Yes','No','Yes');
					if strcmp(answer,'No')
						Continue = 0;
					end
				end
				iteroffsets = 1;
			end
		end
	end
end
if iteroffsets
	XTrilin = zeros(size(x));
	%   options(1) = 0; % No line-search can be performed as iterative offsets are not currently supported in the line-search
end

FeasibilityProblems = 0; % for indicating if the algorithm hasn't yet reached a feasible solution due some constraints
%Define in which mode the scale should go - default is last unless it's fixed')
if all(constraints==-1)
	error(' All modes are fixed, hence no fitting problem')
else
	if constraints(order)==-1 % Since last mode is fixed, scale of the components cannot go there. Instead use the first non-fixed mode
		ScaleMode = min(find(constraints~=-1));
	else
		ScaleMode = order;
	end
end

% CHECK FOR MISSING
if any(isinf(x(:)))|any(isnan(x(:)))
	MissId    = sparse(find(isinf(x)|isnan(x)));
	Missing   = 1;
else
	Missing = 0;
end


% INITIALIZE LOADINGS (if not already given)
if (nargin < 5 | ~(strcmp(class(x0),'cell') | strcmp(class(x0),'struct')))  % Old loadings not given
	x0 = cell(1,order);
	if all(xsize>nocomp) & ~Missing% Use atld
		x0=atld(x,nocomp,0);
		InitString = ' Using fast approximation for initialization';
	elseif order == 3&~Missing&~all(xsize<nocomp)
		% Initialize with TLD estimates
		m = tld(x,nocomp,0,0);
		x0 = m.loads;
		InitString = ' Using direct trilinear decomposition for initialization';
	else
		for j = 1:order
			x0{1,j} = rand(xsize(j),nocomp);
		end
		InitString = ' Using random values for initialization';
	end
else % Old loadings given
	if strcmp(class(x0),'cell')
		x0 = x0;
		InitString = ' Using old values for initialization';
	elseif strcmp(class(x0),'struct')
		if ~strcmp(x0.name,'PARAFAC')
			error(' Input x0 is model that is not a PARAFAC model (name in structure should be PARAFAC)')
		else
			% Use prior model given in x0 to extract loadings and possibly offsets
			if isfield(x0,'Offsets')  % If offsets are given in prior model, then include these
				iteroffsets = 1;
				iteroff = x0.Offsets.Def;
				for i=1:length(iteroff);
					Offset{i}.OffsetsReduced = x0.Offsets.Parameters{1};
					Offset{i}.Offsets = x0.Offsets.Model{1};
				end
				x0 = x0.loads;
				XTrilin = outerm(x0);%zeros(size(x));
				InitString = ' Using old values for initialization (including offsets)';
			else
				x0 = x0.loads;
				InitString = ' Using old values for initialization';
			end
		end
	elseif ~strcmp(class(x0),'cell')
		error('Initial estimate x0 not a cell array')
	end
end


% CHECK FOR INFEASIBLE INITIAL SOLUTIONS
if any(constraints)==1|any(constraints==2) % Nonnegativity required => make sure initial values are positive
	for j=1:order
		if constraints(j)==1|constraints(j)==2
			if any(x0{j}(:)<0) % Negative values occur
				x0{j} = sign(sum(sum(x0{j})))*x0{j}; %'Turn' it around so mainly positive
				i=find(x0{j}<0);
				x0{j}(i)=-x0{j}(i);
			end
		end
	end
end
if any(constraints==3) % Orth required
	for j=1:order
		if constraints(j)==3
			if size(x,j)<nocomp
				error([' Orthogonality cannot be applied in a mode (mode ',num2str(j),') where the dimension is less than the number of factors'])
			else
				x0{j} = orth(x0{j});
			end
		end
	end
end

if Missing % exchange missing elements with model estimates
	xest = outerm(x0);   % Old version;xest = zeros(prod(xsize),nocomp);   for j = 1:nocomp      xvect = x0{1}(:,j);      for ii = 2:order         xvect = xvect*x0{ii}(:,j)';         xvect = xvect(:);      end      xest(:,j) = xvect;   end,   xest = sum(xest,2);   xest = reshape(xest,xsize);
	XTrilin = xest; % Multilinear part to subtract from data before calculating offsets
	if iteroffsets & exist('Offset') % it doesn't exist unless prior model is input
		for i=1:length(iteroff) % estimate each set of offsets one at a time
			xest = xest + Offset{i}.Offsets;
		end
	end
	x(MissId)=xest(MissId);
end
% Calculate total sum of squares in data
if DoWeight
	xsq = (x.*weights).^2;
else
	xsq = x.^2;
end
if Missing
	xsq(MissId)=0;  
end
tssq = sum(xsq(:));

% Initialize the unfolded matrices
xuf = cell(1,order);
xuflo = cell(1,order);
xufsize = zeros(1,order);
for i = 1:order
	xuf{i} = unfoldmw(x,i)';
	xufsize(i) = prod(xsize)/xsize(i);
	% Old init of loads;xuflo{i} = zeros(prod(xsize)/xsize(i),nocomp);
	for j = 1:nocomp
		% Old version if i == 1,         mwvect = x0{2}(:,j);,         for k = 3:order              mwvect = mwvect*x0{k}(:,j)';            mwvect = mwvect(:);         end      else         mwvect = x0{1}(:,j);         for k = 2:order            if k ~= i               mwvect = mwvect*x0{k}(:,j)';               mwvect = mwvect(:);            end         end      end      xuflo{i}(:,j) = mwvect;   end
		xuflo{j}=outerm(x0,j,1);
	end
end

% Initialize other variables needed in the ALS
oldx0 = x0;
searchdir = x0;
iter = 0;
flag = 0;
oldests = zeros(prod(xsize)*nocomp,1);
abschange = 0;
relchange = 0;


% Show algorithmic settings etc.
if DumpToScreen
	disp(' ')
	disp(' Fitting PARAFAC ...') 
	txt=[];
	for i=1:order-1
		txt=[txt num2str(xsize(i)) ' x '];
	end
	txt=[txt num2str(xsize(order))];
	disp([' Input: ',num2str(order),'-way ',txt, ' array'])
	
	disp([' A ',num2str(nocomp),'-component model will be fitted'])
	for i=1:order
		if constraints(i)==0
			disp([' No constraints on mode ',num2str(i)])
		elseif constraints(i)==1
			disp([' Nonnegativity on mode ',num2str(i)])
		elseif constraints(i)==2
			disp([' Unimodality on mode ',num2str(i)])
		elseif constraints(i)==3
			disp([' Orthogonality on mode ',num2str(i)])
		elseif constraints(i)==-1
			disp([' Fixed loadings in mode ',num2str(i)])
		end
	end
	
	disp([' Convergence criteria:']) 
	disp([' Relative change in fit : ',num2str(tol(1))]) 
	disp([' Absolute change in fit : ',num2str(tol(2))]) 
	disp([' Maximum iterations     : ',num2str(tol(3))]) 
	
	if Missing
		disp([' ', num2str(100*(length(MissId)/prod(xsize))),'% missing values']);
	else
		disp(' No missing values')
	end
	
	if DoWeight
		disp(' Weighted optimization will be performed using input weights')
	end
	
	%Iterative preproc
	if iteroffsets
		disp(' Iteratively calculated offsets will be used')
		disp(' Line-search has been turned off')
	else
		%disp(' No iterative offsets')
	end
	
	disp(InitString)
end

% Start the ALS
while flag == 0;
	iter = iter+1;
	% Loop over each of the order to estimate
	
	if DoWeight & iter > 1% If Weighted regression is to be used, do majorization to make a transformed data array to be fitted in a least squares sense
		out = reshape(xest,xsize) + (WMax^(-2)*W2).*(x - reshape(xest,xsize));
		for i = 1:order
			xuf{i} = unfoldmw(out,i)';
		end
		clear out
	end
	
	%Iterative preproc
	if iteroffsets
		if iter>1
			OldOffset = Offset;
		end
		if DoWeight & Iter > 1 % Then the data to fit is not the raw data in x but the modified data according to above in xuf
			xprep = reshape(xuf{1}',xsize) - Xtrilin;
		else
			xprep = x - XTrilin;
		end
		for i=1:length(iteroff) % estimate each set of offsets one at a time
			%[OffsetsFromNstatReduced,OffsetsFromNstat] = nstat(xprep,'mean',iteroff{i},1);
			[OffsetsFromNstatReduced,OffsetsFromNstat] = nstat(xprep,'mean',iteroff{i});
			Offset{i}.OffsetsReduced = OffsetsFromNstatReduced;
			Offset{i}.Offsets = OffsetsFromNstat;
			xprep = xprep - OffsetsFromNstat;
		end
		% Replace unfolded data (xuf), so that the multilinear part is fitted to the residuals of the data subtracted the offsets
		if DoWeight & Iter > 1 % Then the data to fit is not the raw data in x but the modified data according to above in xuf
			xprep = reshape(xuf{1}',xsize);
		else
			xprep = x;
		end
		for i=1:length(iteroff) % estimate each set of offsets one at a time
			xprep = xprep - Offset{i}.Offsets;
		end
		for i = 1:order
			xuf{i} = unfoldmw(xprep,i)';
		end
	end
	
	for i = 1:order
		% Multiply the loads of all the orders together
		% except for the order to be estimated
		xuflo{i}=outerm(x0,i,1);
		
		% Regress the actual data on the estimate to get new loads in order i
		
		% Unconstrained
		if constraints(i) == 0
			ordiest = xuflo{i}\xuf{i};
			
			% nonnegativity   
		elseif constraints(i) == 1 
			ordiest = zeros(nocomp,xsize(i));
			xufloT = xuflo{i}'*xuflo{i}; % Calculate xproduct before loop
			for k = 1:xsize(i)
				ordiest(:,k) = CrossProdFastNnls(xufloT,xuflo{i}'*xuf{i}(:,k));
			end
			if any(sum(ordiest,2)==0);
				FeasibilityProblems=1;
				ordiest = .1*ordiest+.9*x0{i}';
			else
				FeasibilityProblems=0;
			end
			
			%Unimodality
		elseif constraints(i) == 2 
			ordiest=unimodal(xuflo{i},xuf{i},x0{i})';
			if any(sum(ordiest,2)==0);
				FeasibilityProblems=1;
				ordiest = .1*ordiest+.9*x0{i}';
			else
				FeasibilityProblems=0;
			end
			
			%Orthogonality
		elseif constraints(i) == 3
			% if this is the mode holding the scales modify so that loadings are not forced to be length one
			if i == ScaleMode
				Z = [];
				for fac = 1: nocomp
					Z = [Z kron(x0{i}(:,fac)/norm(x0{i}(:,fac)),xuflo{i}(:,fac))];
				end
				Scales = pinv(Z'*Z)*(Z'*xuf{i}(:));
			else
				Scales = ones(1,nocomp);
			end
			ZtX = (xuflo{i}*diag(Scales))'*xuf{i};
			ordiest=((ZtX*ZtX')^(-.5))*ZtX;
			if i == ScaleMode 
				ordiest = diag(Scales)*ordiest;
			end
			
			%Fixed
			
		elseif constraints(i) ~= -1 
			error([' The input constraints has not been correctly defined. The value ',num2str(constraints(i)),' is not possible'])
			
		elseif constraints(i) == -1 
			ordiest = x0{i}';
		end
		
		% Normalize the estimates (except the last (or other defined) order) and store them in the cell
		if i ~= ScaleMode & constraints(i)~=-1 % thus mode fixed
			Scal = 1./sqrt(sum(ordiest.^2,2));
			x0{i} = ordiest'*diag(Scal); % normalization leads to wrong model but that's corrected in the next update of the next mode, and for the last mode no normalization is performed, so that's ok, unless last mode is fixed.
			x0{ScaleMode} = x0{ScaleMode}*diag(Scal.^(-1));%ii = i+1;
		else
			x0{i} = ordiest';%ii = 1;
		end
		
	end
	% Calculate the estimate of the input array based on current loads
	xest = outerm(x0);
	% old version; xest = zeros(prod(xsize),nocomp);for j = 1:nocomp,      xvect = x0{1}(:,j);,      for ii = 2:order,         xvect = xvect*x0{ii}(:,j)';         xvect = xvect(:);      end,      xest(:,j) = xvect;,   end,   xest = sum(xest,2);   xest = reshape(xest,xsize);
	%Iterative preproc
	if iteroffsets
		XTrilin = xest; % Multilinear part to subtract from data before calculating offsets
		for ii=1:length(iteroff) % Add offsets one at a time
			xest = xest + Offset{ii}.Offsets;
		end      
	end
	xsq = xest;
	% Exchange missing with model estimates
	if Missing 
		x(MissId)=xest(MissId);
		for ii = 1:order
			xuf{ii} = unfoldmw(x,ii)';
		end
	end
	% Check to see if the fit has changed significantly
	if DoWeight
		xsq = ((x-xsq).*weights).^2;
	else
		xsq = (x-xsq).^2;
	end
	if Missing
		xsq(MissId)=0;  
	end
	ssq = sum(xsq(:));
	
	
	%disp(sprintf('On iteration %g ALS fit = %g',iter,ssq));
	if iter > 1 &~FeasibilityProblems
		abschange = abs(oldssq-ssq);
		relchange = abschange/ssq;
		if relchange < tol(1)
			flag = 1;
			if DumpToScreen
				disp(' '),disp('    Iteration    Rel. Change         Abs. Change         sum-sq residuals'),disp(' ')
				fprintf(' %9.0f       %12.10f        %12.10f        %12.10f    \n',iter,relchange,abschange,ssq);
				disp(' ')
				disp(' Iterations terminated based on relative change in fit error')
			end
		elseif abschange < tol(2)
			flag = 1;
			if DumpToScreen
				disp(' '),disp('    Iteration    Rel. Change         Abs. Change         sum-sq residuals'),disp(' ')
				fprintf(' %9.0f       %12.10f        %12.10f        %12.10f    \n',iter,relchange,abschange,ssq);
				disp(' ')
				disp(' Iterations terminated based on absolute change in fit error')
			end
		elseif iter > tol(3)-1
			flag = 1;
			if DumpToScreen
				disp(' '),disp('    Iteration    Rel. Change         Abs. Change         sum-sq residuals'),disp(' ')
				fprintf(' %9.0f       %12.10f        %12.10f        %12.10f    \n',iter,relchange,abschange,ssq);
				disp(' ')
				disp(' Iterations terminated based on maximum iterations')
			end
		end
	end
	if rem(iter,Show) == 0&DumpToScreen
		if iter == Show|rem(iter,Show*30) == 0
			disp(' '),disp('    Iteration    Rel. Change         Abs. Change         sum-sq residuals'),disp(' ')
		end
		fprintf(' %9.0f       %12.10f        %12.10f        %12.10f    \n',iter,relchange,abschange,ssq);
	end
	oldssq = ssq;
	
	% Every fifth iteration do a line search if ls == 1
	if (iter/5 == round(iter/5) & ls == 1)
		% Determine the search direction as the difference between the last two estimates
		for ij = 1:order
			searchdir{ij} = x0{ij} - oldx0{ij};
		end
		if iteroffsets
			for ij=1:length(iteroff)
				searchdirOffs{ij} = Offset{ij}.Offsets - OldOffset{ij}.Offsets;
			end
		end      
		% Initialize other variables required for line search
		testmod = x0;
		sflag = 0; 
		i = 0; 
		sd = zeros(10,1); 
		sd(1) = ssq;
		xl = zeros(10,1);
		while sflag == 0
			for k = 1:order
				testmod{k} = testmod{k} + (2^i)*searchdir{k};
			end
			if iteroffsets
				testoffsets = Offset;
				for j=1:length(Offset)
					testoffsets{j}.Offsets = testoffsets{j}.Offsets + (2^i)*searchdirOffs{j};
				end
			end
			% Calculate the fit error on the new test model
			xest = outerm(testmod);
			%Iterative preproc
			if iteroffsets
				XTrilin = xest; % TO SUBTRACT FROM DATA 
				for j=1:length(iteroff) % Add offsets one at a time
					xest = xest + testoffsets{j}.Offsets;
				end
			end
			if DoWeight
				xsq = ((x-outerm(testmod)).*weights).^2;
			else
				xsq = (x-outerm(testmod)).^2;
			end
			if Missing
				xsq(MissId)=0;  
			end
			% Save the difference and the distance along the search direction
			sd(i+2) = sum(xsq(:));
			xl(i+2) = xl(i+1) + 2^i;
			i = i+1;
			% Check to see if a minimum has been exceeded once two new points are calculated
			if i > 1 
				if sd(i+1) > sd(i)
					sflag = 1;
					% Estimate the minimum along the search direction
					xstar = sum((xl([i i+1 i-1]).^2 - xl([i+1 i-1 i]).^2).*sd(i-1:i+1));
					xstar = xstar/(2*sum((xl([i i+1 i-1]) - xl([i+1 i-1 i])).*sd(i-1:i+1)));
					% Save the old model and update the new one
					oldx0 = x0;
					for k = 1:order
						x0{k} = x0{k} + xstar*searchdir{k};
					end
					if iteroffsets
						OldOffset = Offset;
						for j=1:length(iteroff)
							Offset{j}.Offsets = Offset{j}.Offsets + + xstar*searchdirOffs{j};
						end
					end      
					
				end
			end
		end 
		% Calculate the estimate of the input array based on current loads
		xest = outerm(x0);
		%Iterative preproc
		if iteroffsets
			XTrilin = xest; % TO SUBTRACT FROM DATA 
			for j=1:length(iteroff) % Add offsets one at a time
				xest = xest + Offset{j}.Offsets;
			end      
		end
		xsq = xest;
		if DoWeight
			xsq = ((x-xsq).*weights).^2;
		else
			xsq = (x-xsq).^2;
		end
		if Missing
			xsq(MissId)=0;  
		end
		oldssq = sum(xsq(:));
		if Missing 
			x(MissId)=xest(MissId);
			for j = 1:order
				xuf{j} = unfoldmw(x,j)';
			end
		end
		% disp(sprintf('SSQ at xstar is %g',oldssq))
	else
		% Save the last estimates of the loads
		oldx0 = x0;
	end
	% Exchange missing with model estimates
end


% Plot the loadings
if plots ~= 0
	h1 = figure('position',[170 130 512 384],'name','PARAFAC Loadings');
	for i = 1:order
		subplot(order,1,i)
		if ~isempty(scl{i})
			if xsize(i) <= 50
				plot(scl{i},x0{i},'-+')
			else
				plot(scl{i},x0{i},'-')
			end
		else
			if xsize(i) <= 50
				plot(x0{i},'-+')
			else
				plot(x0{i},'-')
			end
		end
		ylabel(sprintf('Dimension %g',i))
		if i == 1
			title('Loadings for Each Dimension')
		end
	end
end

% Calculate and plot the residuals  
dif = (x-outerm(x0)).^2;
res = cell(1,order);
for i = 1:order
	x = dif;
	for j = 1:order
		if i ~= j
			x = sum(x,j);
		end
	end
	x = squeeze(x);
	res{i} = x(:);
	if plots ~= 0
		if i == 1  
			figure('position',[145 166 512 384],'name','PARAFAC Residuals')
		end
		subplot(order,1,i)
		if ~isempty(scl{i})
			if xsize(i) <= 50
				plot(scl{i},res{i},'-+')
			else
				plot(scl{i},res{i},'-')
			end
		else
			if xsize(i) <= 50
				plot(res{i},'-+')
			else
				plot(res{i},'-')
			end
		end
		ylabel(sprintf('Dimension %g',i))
		if i == 1
			title('Residuals for Each Dimension')
		end
	end
end

% Bring the loads back to the front
if plots ~= 0
	figure(h1)
end

% Save the model as a structured array    
model = struct('xname',inputname(1),'name','PARAFAC','date',date,'time',clock,...
	'size',xsize,'nocomp',nocomp,'tol',tol,'final',[relchange abschange iter]);
model.ssq = [tssq ssq];
model.loads = x0;
model.res = res;
model.scl = scl;
%Iterative preproc
if iteroffsets
	for i=1:length(iteroff) % Add offsets one at a time
		model.Offsets.Parameters{i} = Offset{i}.OffsetsReduced;
		model.Offsets.Model{i} = Offset{i}.Offsets;
		model.Offsets.Def{i} = iteroff{i};
	end      
end

function B=unimodal(X,Y,Bold)

% Solves the problem min|Y-XB'| subject to the columns of 
% B are unimodal and nonnegative. The algorithm is iterative
% If an estimate of B (Bold) is given only one iteration is given, hence
% the solution is only improving not least squares
% If Bold is not given the least squares solution is estimated
%
% Copyright 1997
%
% Rasmus Bro
% Royal Veterinary & Agricultural University
% Denmark
% rb@kvl.dk
%
% Reference
% Bro and Sidiropoulos, "Journal of Chemometrics", 1998, 12, 223-247. 


if nargin==3
	B=Bold;
	F=size(B,2);
	for f=1:F
		y=Y-X(:,[1:f-1 f+1:F])*B(:,[1:f-1 f+1:F])';
		beta=pinv(X(:,f))*y;
		B(:,f)=ulsr(beta',1);
	end
else
	F=size(X,2);
	maxit=100;
	B=randn(size(Y,2),F);
	Bold=2*B;
	it=0;
	while norm(Bold-B)/norm(B)>1e-5&it<maxit
		Bold=B;
		it=it+1;
		for f=1:F
			y=Y-X(:,[1:f-1 f+1:F])*B(:,[1:f-1 f+1:F])';
			beta=pinv(X(:,f))*y;
			B(:,f)=ulsr(beta',1);
		end
	end
	if it==maxit
		disp([' UNIMODAL did not converge in ',num2str(maxit),' iterations']);
	end
end


function [b,All,MaxML]=ulsr(x,NonNeg);

% ------INPUT------
%
% x          is the vector to be approximated
% NonNeg     If NonNeg is one, nonnegativity is imposed
%
%
%
% ------OUTPUT-----
%
% b 	     is the best ULSR vector
% All 	     is containing in its i'th column the ULSRFIX solution for mode
% 	     location at the i'th element. The ULSR solution given in All
%            is found disregarding the i'th element and hence NOT optimal
% MaxML      is the optimal (leftmost) mode location (i.e. position of maximum)
%
% ___________________________________________________________
%
%
%               Copyright 1997
%
% Nikos Sidiroupolos
% University of Maryland
% Maryland, US
%
%       &
%
% Rasmus Bro
% Royal Veterinary & Agricultural University
% Denmark
%
% 
% ___________________________________________________________


% This file uses MONREG.M

x=x(:);
I=length(x);
xmin=min(x);
if xmin<0
	x=x-xmin;
end


% THE SUBSEQUENT 
% CALCULATES BEST BY TWO MONOTONIC REGRESSIONS

% B1(1:i,i) contains the monontonic increasing regr. on x(1:i)
[b1,out,B1]=monreg(x);

% BI is the opposite of B1. Hence BI(i:I,i) holds the monotonic
% decreasing regression on x(i:I)
[bI,out,BI]=monreg(flipud(x));
BI=flipud(fliplr(BI));

% Together B1 and BI can be concatenated to give the solution to
% problem ULSR for any modloc position AS long as we do not pay
% attention to the element of x at this position


All=zeros(I,I+2);
All(1:I,3:I+2)=B1;
All(1:I,1:I)=All(1:I,1:I)+BI;
All=All(:,2:I+1);
Allmin=All;
Allmax=All;
% All(:,i) holds the ULSR solution for modloc = i, disregarding x(i),


iii=find(x>=max(All)');
b=All(:,iii(1));
b(iii(1))=x(iii(1));
Bestfit=sum((b-x).^2);
MaxML=iii(1);
for ii=2:length(iii)
	this=All(:,iii(ii));
	this(iii(ii))=x(iii(ii));
	thisfit=sum((this-x).^2);
	if thisfit<Bestfit
		b=this;
		Bestfit=thisfit;
		MaxML=iii(ii);
	end
end

if xmin<0
	b=b+xmin;
end


% Impose nonnegativity
if NonNeg==1
	if any(b<0)
		id=find(b<0);
		% Note that changing the negative values to zero does not affect the
		% solution with respect to nonnegative parameters and position of the
		% maximum.
		b(id)=zeros(size(id))+0;
	end
end



function [b,B,AllBs]=monreg(x);

% Monotonic regression according
% to J. B. Kruskal 64
%
% b     = min|x-b| subject to monotonic increase
% B     = b, but condensed
% AllBs = All monotonic regressions, i.e. AllBs(1:i,i) is the 
%         monotonic regression of x(1:i)
%
%
% Copyright 1997
%
% Rasmus Bro
% Royal Veterinary & Agricultural University
% Denmark
% rb@kvl.dk
%


I=length(x);
if size(x,2)==2
	B=x;
else
	B=[x(:) ones(I,1)];
end

AllBs=zeros(I,I);
AllBs(1,1)=x(1);
i=1;
while i<size(B,1)
	if B(i,1)>B(min(I,i+1),1)
		summ=B(i,2)+B(i+1,2);
		B=[B(1:i-1,:);[(B(i,1)*B(i,2)+B(i+1,1)*B(i+1,2))/(summ) summ];B(i+2:size(B,1),:)];
		OK=1;
		while OK
			if B(i,1)<B(max(1,i-1),1)
				summ=B(i,2)+B(i-1,2);
				B=[B(1:i-2,:);[(B(i,1)*B(i,2)+B(i-1,1)*B(i-1,2))/(summ) summ];B(i+1:size(B,1),:)];
				i=max(1,i-1);
			else
				OK=0;
			end
		end
		bInterim=[];
		for i2=1:i
			bInterim=[bInterim;zeros(B(i2,2),1)+B(i2,1)];
		end
		No=sum(B(1:i,2));
		AllBs(1:No,No)=bInterim;
	else
		i=i+1;
		bInterim=[];
		for i2=1:i
			bInterim=[bInterim;zeros(B(i2,2),1)+B(i2,1)];
		end
		No=sum(B(1:i,2));
		AllBs(1:No,No)=bInterim;
	end
end

b=[];
for i=1:size(B,1)
	b=[b;zeros(B(i,2),1)+B(i,1)];
end


function [x,w] = CrossProdFastNnls(XtX,Xty,tol)
%NNLS	Non-negative least-squares.
%	b = CrossProdFastNnls(XtX,Xty) returns the vector b that solves X*b = y
%	in a least squares sense, subject to b >= 0, given the inputs
%       XtX = X'*X and Xty = X'*y.
%
%	[b,w] = fastnnls(XtX,Xty) also returns dual vector w where
%	w(i) < 0 where b(i) = 0 and w(i) = 0 where b(i) > 0.
%	L. Shure 5-8-87 Copyright (c) 1984-94 by The MathWorks, Inc.
%
%  Revised by:
%	Copyright
%	Rasmus Bro 1995
%	Denmark
%	E-mail rb@kvl.dk
%  According to Bro & de Jong, J. Chemom, 1997, 11, 393-401

% initialize variables


if nargin < 3
	tol = 10*eps*norm(XtX,1)*max(size(XtX));
end
[m,n] = size(XtX);
P = zeros(1,n);
Z = 1:n;
x = P';
ZZ=Z;
w = Xty-XtX*x;

% set up iteration criterion
iter = 0;
itmax = 30*n;

% outer loop to put variables into set to hold positive coefficients
while any(Z) & any(w(ZZ) > tol)
	[wt,t] = max(w(ZZ));
	t = ZZ(t);
	P(1,t) = t;
	Z(t) = 0;
	PP = find(P);
	ZZ = find(Z);
	nzz = size(ZZ);
	z(PP')=(Xty(PP)'/XtX(PP,PP)');
	z(ZZ) = zeros(nzz(2),nzz(1))';
	z=z(:);
	% inner loop to remove elements from the positive set which no longer belong
	
	while any((z(PP) <= tol)) & iter < itmax
		
		iter = iter + 1;
		QQ = find((z <= tol) & P');
		alpha = min(x(QQ)./(x(QQ) - z(QQ)));
		x = x + alpha*(z - x);
		ij = find(abs(x) < tol & P' ~= 0);
		Z(ij)=ij';
		P(ij)=zeros(1,max(size(ij)));
		PP = find(P);
		ZZ = find(Z);
		nzz = size(ZZ);
		z(PP)=(Xty(PP)'/XtX(PP,PP)');
		z(ZZ) = zeros(nzz(2),nzz(1));
		z=z(:);
	end
	x = z;
	w = Xty-XtX*x;
end

x=x(:);









function loads=atld(X,F,Show);



if nargin<3
	Show = 0;
end

xsize = size(X);
order = ndims(X);
RealData = all(isreal(X(:)));

% Initialize with random numbers
for i = 1:order;
	if xsize(i)>=F
		loads{i} = orth(rand(xsize(i),F));
	else
		loads{i} = rand(xsize(i),F);
	end
	invloads{i} = pinv(loads{i});
end

model = outerm(loads);
fit=sum(abs((X(:)-model(:)).^2));
oldfit=2*fit;
maxit=30;
crit=1e-6;
it=0;

while abs(fit-oldfit)/oldfit>crit&it<maxit
	it=it+1;
	oldfit=fit;
	
	% Normalize loadings   
	for mode = 2:order
		scale = sqrt(abs(sum(loads{mode}.^2)));
		loads{1}    = loads{1}*diag(scale);
		loads{mode} = loads{mode}*diag(scale.^-1);
		% Scale occasionally to mainly positiviy
		if RealData & (it==1|it==2|rem(it,1)==0)
			scale = sign(sum(loads{mode}));
			loads{1}    = loads{1}*diag(scale);
			loads{mode} = loads{mode}*diag(scale);
			invloads{mode} = pinv(loads{mode});
		end
	end
	
	if it == 1
		delta = linspace(1,F^(order-1),F);
	end
	
	% Compute new loadings for all modes
	for mode = 1:order;
		
		xprod = X;
		% Multiply pseudo-inverse loadings in all but the mode being estimated
		if mode == 1
			for mulmode = 2:order
				xprod = ntimes(xprod,invloads{mulmode},2,2);
			end
		else
			for j = 1:mode-1
				xprod = ntimes(xprod,invloads{j},1,2);
			end
			for j = mode+1:order
				xprod = ntimes(xprod,invloads{j},2,2);
			end
		end
		
		% Extract first mode loadings from product
		loads{mode} = xprod(:,delta);
		invloads{mode} = pinv(loads{mode});
	end
	
	
	model = outerm(loads);
	fit=sum((X(:)-model(:)).^2);
end
% Normalize loadings   
for mode = 2:order
	scale = sqrt(sum(loads{mode}.^2));
	loads{1}    = loads{1}*diag(scale);
	loads{mode} = loads{mode}*diag(scale.^-1);
end

if Show
	disp(' ')
	disp('    Iteration    sum-sq residuals')
	disp(' ')
	fprintf(' %9.0f       %12.10f    \n',it,fit);
end




function product = ntimes(X,Y,modeX,modeY);


%NTIMES Array multiplication
% X*Y is the array/matrix product of X and Y. These are multiplied across the 
% modeX mode/dimension of X and modeY mode/dimension of Y. The number of levels of X in modeX
% must equal the number of levels in modeY of Y.
%
% The product will be an array of order two less than the sum of the orders of A and B
% and thus works as a straightforward extension of the matrix product. The order
% of the modes in the product are such that the X-modes come first and then the
% Y modes. 
%
% E.g. if X is IxJ and Y is KxL and I equals L, then 
% ntimes(X,Y,1,2) will yield a JxK matrix equivalent to the matrix product
% X'*Y'. If X is IxJxK and Y is LxMxNxO with J = N then 
% ntimes(X,Y,2,3) yields a product of size IxKxLxMxO
% 
%I/O: product = NTIMES(X,Y,modeX,modeY) 
% 
%See also TIMES,MTIMES.

%Copyright Eigenvector Research Inc./Rasmus Bro, 2000
%Rasmus Bro, August 20, 2000

orderX = ndims(X);
orderY = ndims(Y);
xsize  = size(X);
ysize  = size(Y);

X = permute(X,[modeX 1:modeX-1 modeX+1:orderX]);
Y = permute(Y,[modeY 1:modeY-1 modeY+1:orderY]);
xsize2  = size(X);
ysize2  = size(Y);

if size(X,1)~=size(Y,1)
	error(' The number of levels must be the same in the mode across which multiplications is performed')
end

%multiply the matricized arrays
product = reshape(X,xsize2(1),prod(xsize2(2:end)))'*reshape(Y,ysize2(1),prod(ysize2(2:end)));
%reshape to three-way structure
product = reshape(product,[xsize2(2:end) ysize2(2:end)]);






function mwa = outerm(facts,lo,vect)
if nargin < 2
  lo = 0;
end
if nargin < 3
  vect = 0;
end
order = length(facts);
if lo == 0
  mwasize = zeros(1,order);
else
  mwasize = zeros(1,order-1);
end
k = 0;
for i = 1:order
  [m,n] = size(facts{i});
  if i ~= lo
    k = k + 1;
    mwasize(k) = m;
  end
  if i > 1
    if nofac ~= n
	  error('All orders must have the same number of factors')
	end
  else
    nofac = n;
  end
end
mwa = zeros(prod(mwasize),nofac);

for j = 1:nofac
  if lo ~= 1
    mwvect = facts{1}(:,j);
    for i = 2:order
	  if lo ~= i
        %mwvect = kron(facts{i}(:,j),mwvect);
		mwvect = mwvect*facts{i}(:,j)';
		mwvect = mwvect(:);
	  end
    end
  elseif lo == 1
    mwvect = facts{2}(:,j);
	for i = 3:order
      %mwvect = kron(facts{i}(:,j),mwvect);
	  mwvect = mwvect*facts{i}(:,j)';
	  mwvect = mwvect(:);
	end
  end
  mwa(:,j) = mwvect;
end
% If vect isn't one, sum up the results of the factors and reshape
if vect ~= 1
  mwa = sum(mwa,2);
  mwa = reshape(mwa,mwasize);
end

function mwauf = unfoldmw(mwa,order)
mwasize = size(mwa);
ms = mwasize(order);
po = prod(mwasize);
ns = po/ms;
if order ~= 1
  pod = prod(mwasize(1:order-1));
end
mwauf = zeros(ms,ns);
for i = 1:ms
  if order == 1
    mwauf(i,:) = mwa(i:ms:po);
  else
    inds = zeros(1,ns); k = 1; fi = (i-1)*pod + 1;
	for j = 1:ns/pod
	  inds(k:k+pod-1) = fi:fi+pod-1;
	  fi = fi + ms*pod;
	  k = k + pod;
	end
	mwauf(i,:) = mwa(inds);
  end
end
