function [T,P,offset,Loss,Count]=milespca(x,weights,Fac,Cent,FixP,FixOffset);

%MILES-PCA for fitting maximum likelihood PCA
%
% INPUT:
% x         IxJ data matrix 
% 
% weights   matrix of size IJ x IJ holding the weights. For computational
%           reasons, it is advisable to provide the weights in a sparse 
%           matrix if possible.
%           
%           If the weights provided are only of size JxJ, they are assumed to be
%           independent and identical across rows.
%
% Fac       Number of factors to use
% 
% Cent      If Cent = 1, the data are centered, else not
% 
% optional inputs
% FixP      Earlier found loadings. Used for fitting new data to estimated model
%
% FixOffset Earlier found loadings. Used for fitting new data to estimated model
%
% 
% 
% OUTPUT:
% 
% I/O: [T,P,offset,Loss]=milespca(x,weights,Fac,Cent,FixP,FixOffset);
%
%
% Copyright Rasmus Bro, Nikos Sidiropoulos, Age Smilde, 2000
%           rb@kvl.dk
%
% If you use this m-file please supply reference to 
% Rasmus Bro, Nicholas D. Sidiropoulos, Age K. Smilde
% Maximum likelihood fitting using ordinary least squares algorithms
% Journal of Chemometrics, 2001, Submitted



% Initialization
ConvCrit = 1e-11;
ShowFit  = 500; 
MaxIt    = 10000;
if nargin<4,Cent = 0;end
if nargin<5,FixOffset=0;FixP=0;end
DimX = size(x);
wtw = weights'*weights;
model    = x;
residual = zeros(size(x));
Loss0 = mlloss(x,wtw,DimX(2));
Loss     = Loss0;
OldLoss  = Loss+1;
Count    = 0;
LineSearchDelta = 20;

clear weights
OPTIONS.disp = 0;
beta = eigs(wtw,1,'LM',OPTIONS);
   
   
lort=[];

while abs(Loss-OldLoss)/OldLoss > ConvCrit & Count < MaxIt
   
   Count = Count + 1;
   OldLoss = Loss;
   
   if Count>1
      OldQ = Q;
   end
   if size(wtw,2)==DimX(2)  % Covariance across rows zero
      Q = model + (1/beta)*residual*wtw;
   else
      q = model(:) + (1/beta)*wtw*residual(:);
      Q = reshape(q,DimX(1),DimX(2));
   end
   
   % Quadratic line search
   if ~rem(Count,4)&Count>20
      Del1 = LineSearchDelta;Del2 = LineSearchDelta*2;  % Base the linesearch on old value as well as old optimal delta (+ 2*delta)
      DeltaQ = Q-OldQ;
      loss1 = Loss;
      [t,p,offset,modells] = pca(Q+Del1*DeltaQ,Fac,Cent,FixP,FixOffset);
      residualls = x - modells;
      loss2 = mlloss(residualls,wtw,DimX(2));
      [t,p,offset,modells] = pca(Q+Del2*DeltaQ,Fac,Cent,FixP,FixOffset);
      residualls = x - modells;
      loss3 = mlloss(residualls,wtw,DimX(2));
      beta23 = Del1^2-Del2^2;beta31=Del2^2;beta12=-Del1^2;gamma23=Del1-Del2;gamma31=Del2;gamma12=-Del1;
      
      % 'Optimal' delta
      LineSearchDelta = .5 *(beta23*loss1+beta31*loss2+beta12*loss3) / (gamma23*loss1+gamma31*loss2+gamma12*loss3);
      if isnan(LineSearchDelta)|isinf(LineSearchDelta)
         LineSearchDelta = 10;
      end
      
      [t,p,offset,modells] = pca(Q+LineSearchDelta*DeltaQ,Fac,Cent,FixP,FixOffset);
      residualls = x - modells;
      Lossls = mlloss(residualls,wtw,DimX(2));
      
      if Lossls<Loss
         Loss = Lossls;
         model = modells;
         T=t;P=p;
         if size(wtw,2)==DimX(2)  % Covariance across rows zero
            Q = modells + (1/beta)*residual*wtw;
         else
            q = modells(:) + (1/beta)*wtw*residual(:);
            Q = reshape(q,DimX(1),DimX(2));
         end
      end
   end

      
   
   [T,P,offset,model] = pca(Q,Fac,Cent,FixP,FixOffset);
   residual = x - model;
   Loss = mlloss(residual,wtw,DimX(2));
   lort=[lort Loss]; 
   if rem(Count,ShowFit)==0
      fprintf(' %8.0f       %12.10f        %12.10f       \n',Count,Loss,100*(1-Loss/Loss0));
      if Count>1,   plot(log(lort(2:end))),drawnow,end
   end
   
end
fprintf(' %8.0f       %12.10f        %12.10f       \n',Count,Loss,100*(1-Loss/Loss0));

function [t,p,offset,model] = pca(X,Fac,Cent,FixP,FixOffset)

% Center if necessary
if Cent==1   
   % Calculate offsets
   if length(FixOffset)~=size(X,2)
      offset = mean(X);
   else  % Or use those provided
      offset = FixOffset;
   end
   X = X - ones(size(X,1),1)*offset;
else
   offset = zeros(1,size(X,2));
end

% Find pca model
if length(FixOffset)~=size(X,2) 
   [u,s,v] = svd(X,0);
   t = u(:,1:Fac)*s(1:Fac,1:Fac);
   p = v(:,1:Fac);
else      % Use prior given P
   t = X*FixP;
   p = FixP;
end

% Calculate model
model = t*p' + ones(size(X,1),1)*offset;


function loss = mlloss(residual,wtw,J)

if size(wtw,2)==J  % Covariance across rows zero
   loss = sum(sum((residual*wtw).*residual));
else
   loss = residual(:)'*wtw*residual(:);
end